//+------------------------------------------------------------------+
//|                                                          TVI.mq4 |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1  DeepSkyBlue
#property indicator_width1  2

//
//
//
//
//

extern string TimeFrame   = "current time frame";
extern int    TviPeriod1  = 12;
extern int    TviPeriod2  = 12;
extern int    TviPeriod3  =  5;
extern bool   Interpolate = true;

//
//
//
//
//

double Tvi[];
double TviBuffer[];
double uTi[];
double dTi[];
double Ema1[];
double Ema2[];
double Emaa1[];
double Emaa2[];

//
//
//
//
//

int    timeFrame;
string indicatorFileName;
bool   calculating;
bool   returningBars;


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(8);
      SetIndexBuffer(0,Tvi); SetIndexLabel(1,"TVI");
      SetIndexBuffer(1,TviBuffer);
      SetIndexBuffer(2,uTi);
      SetIndexBuffer(3,dTi);
      SetIndexBuffer(4,Ema1);
      SetIndexBuffer(5,Ema2);
      SetIndexBuffer(6,Emaa1);
      SetIndexBuffer(7,Emaa2);
      
      //
      //
      //
      //
      //
      
         indicatorFileName = WindowExpertName();
         calculating       = (TimeFrame=="calculateTVI"); if (calculating)   return(0);
         returningBars     = (TimeFrame=="returnBars");   if (returningBars) return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);
         
      //
      //
      //
      //
      //
               
   IndicatorShortName("TVI "+timeFrameToString(timeFrame)+" ("+TviPeriod1+","+TviPeriod2+","+TviPeriod3+")");
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returningBars)  { Tvi[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (calculating || timeFrame==Period())
   {   
      double alfa1 = 2.0 / (1.0 + TviPeriod1);
      double alfa2 = 2.0 / (1.0 + TviPeriod2);
      double alfaS = 2.0 / (1.0 + TviPeriod3);

      for (i=limit; i>=0; i--)
      {
            uTi[i] = (Volume[i]+(Close[i]-Open[i])/Point)/2.0;
            dTi[i] = (Volume[i]-uTi[i]);

                  Ema1[i]  = Ema1[i+1]  + alfa1*(uTi[i]   - Ema1[i+1]);
                  Ema2[i]  = Ema2[i+1]  + alfa2*(Ema1[i]  - Ema2[i+1]);
                  Emaa1[i] = Emaa1[i+1] + alfa1*(dTi[i]   - Emaa1[i+1]);
                  Emaa2[i] = Emaa2[i+1] + alfa2*(Emaa1[i] - Emaa2[i+1]);
            
            if ((Emaa2[i]+Emaa2[i]) != 0)
                  TviBuffer[i] = 100.0*(Ema2[i]-Emaa2[i])/(Ema2[i]+Emaa2[i]);
            else  TviBuffer[i] = 0.00;
            Tvi[i] = Tvi[i+1] + alfaS*(TviBuffer[i]-Tvi[i+1]);
      }
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         Tvi[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateTVI",TviPeriod1,TviPeriod2,TviPeriod3,0,y);

         //
         //
         //
         //
         //
      
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            double factor = 1.0 / n;
            for(int k = 1; k < n; k++)
               Tvi[i+k] = k*factor*Tvi[i+n] + (1.0-k*factor)*Tvi[i];
   }
   
   return(0);      
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}